#include <GL/glew.h>
#if defined(_WIN32)
#  include <GL/wglew.h>
#elif !defined(__APPLE__) || defined(GLEW_APPLE_GLX)
#  include <GL/glxew.h>
#endif

/*
* Define glewGetContext and related helper macros.
*/
#ifdef GLEW_MX
#  define glewGetContext() ctx
#  ifdef _WIN32
#    define GLEW_CONTEXT_ARG_DEF_INIT GLEWContext* ctx
#    define GLEW_CONTEXT_ARG_VAR_INIT ctx
#    define wglewGetContext() ctx
#    define WGLEW_CONTEXT_ARG_DEF_INIT WGLEWContext* ctx
#    define WGLEW_CONTEXT_ARG_DEF_LIST WGLEWContext* ctx
#  else /* _WIN32 */
#    define GLEW_CONTEXT_ARG_DEF_INIT void
#    define GLEW_CONTEXT_ARG_VAR_INIT
#    define glxewGetContext() ctx
#    define GLXEW_CONTEXT_ARG_DEF_INIT void
#    define GLXEW_CONTEXT_ARG_DEF_LIST GLXEWContext* ctx
#  endif /* _WIN32 */
#  define GLEW_CONTEXT_ARG_DEF_LIST GLEWContext* ctx
#else /* GLEW_MX */
#  define GLEW_CONTEXT_ARG_DEF_INIT void
#  define GLEW_CONTEXT_ARG_VAR_INIT
#  define GLEW_CONTEXT_ARG_DEF_LIST void
#  define WGLEW_CONTEXT_ARG_DEF_INIT void
#  define WGLEW_CONTEXT_ARG_DEF_LIST void
#  define GLXEW_CONTEXT_ARG_DEF_INIT void
#  define GLXEW_CONTEXT_ARG_DEF_LIST void
#endif /* GLEW_MX */

#ifdef _WIN32
typedef void*(WINAPI *GPA)(const GLubyte*);
typedef void*(WINAPI *_GPA)(const char*);
typedef  PROC  (WINAPI* WGLGPA)(LPCSTR);
static WGLGPA _wglewGetProcAddress = NULL;
#else
typedef void*(*GPA)(const GLubyte*);
typedef void*(*_GPA)(const char*);
#endif

static enum GL_Name_Convention glew_convention = GLEW_NAME_STANDARD;
static enum GL_Library_Type gl_lib_type = GLEW_LIB_TYPE_NATIVE;
#ifdef _WIN32
static const char *glew_gl_lib = "opengl32.dll";
#elif defined(__APPLE__)
static const char *glew_gl_lib = "/System/Library/Frameworks/OpenGL.framework/Versions/Current/OpenGL";
#elif defined(_AIX)
static const char *glew_gl_lib = "libGL.a";
#elif defined(__linux__)
static const char *glew_gl_lib = "libGL.so.1";
#endif

static GPA __glewXGetProcAddress = NULL;

#if defined(__APPLE__)
#include <stdlib.h>
#include <string.h>
#include <AvailabilityMacros.h>

#ifdef MAC_OS_X_VERSION_10_3

#include <dlfcn.h>

void* NSGLGetProcAddress (const GLubyte *name)
{
  static void* image = NULL;
  if (NULL == image)
  {
    image = dlopen("/System/Library/Frameworks/OpenGL.framework/Versions/Current/OpenGL", RTLD_LAZY);
  }
  return image ? dlsym(image, (const char*)name) : NULL;
}

#endif /* MAC_OS_X_VERSION_10_3 */
#endif /* __APPLE__ */

#if defined(__sgi) || defined (__sun)
#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>

void* dlGetProcAddress (const GLubyte* name)
{
  static void* h = NULL;
  static void* gpa;

  if (h == NULL)
  {
    if ((h = dlopen(NULL, RTLD_LAZY | RTLD_LOCAL)) == NULL) return NULL;
    gpa = dlsym(h, "glXGetProcAddress");
  }

  if (gpa != NULL)
    return ((void*(*)(const GLubyte*))gpa)(name);
  else
    return dlsym(h, (const char*)name);
}
#endif /* __sgi || __sun */

#if defined(__linux__) || defined(__APPLE__) || defined(_WIN32) || \
    defined(_AIX)
#if defined (_WIN32)
static HMODULE dso_handle = NULL;
#else
#include <dlfcn.h>
static void *dso_handle = NULL;
#endif

#if defined(__GNUC__) && __GNUC__ >= 4
# define UNUSED __attribute__((unused))
#else
# define UNUSED /* no attribute support */
#endif

/* Give a couple debugging macros which allow us to report issues, but only if
* a debugging define is given.  When defined, GLEW requires the standard C
* library. */
#if defined(GLEW_DEBUG) && defined(__STDC_VERSION__) && \
    __STDC_VERSION__ >= 199901L
# include <stdio.h>
# include <stdlib.h>
# define warning(...) do { fprintf(stderr, __VA_ARGS__); } while(0)
# define error(...) do { fprintf(stderr, __VA_ARGS__); abort(); } while(0)
#else
  /* Some versions of IBM's xlc do not handle an ellipsis for macro arguments,
   * even if the macro definition doesn't use __VA_ARGS__.  Thus we define
   * empty functions for these instead of empty macros. */
  UNUSED static void warning(UNUSED const char* s, ...) { }
  static void error(UNUSED const char* s, ...) { }
#endif

/* Function pointer for OSMesa function loader. */
static _GPA _glewOSMesaGetProcAddress = NULL;

/* The interfaces to GetProcAddress for glX and OSMesa differ in both return
* type and argument type.  To get around this, we create two thunks which
* forward to the appropriate function, casting back and forth. */
static void* _glew_glx_thunk_gpa(const char *name) {
  return (void*) __glewXGetProcAddress((const GLubyte*)name);
}
#ifdef __APPLE__
/* Apple doesn't use a function loader in its proprietary APIs: just load the
 * symbol directly from the object. */
static void* _glew_dlsym_thunk_gpa(const char *name) {
  return (void*) dlsym(dso_handle, name);
}
#endif
static void* _glew_osmesa_thunk_gpa(const char *name) {
  return _glewOSMesaGetProcAddress(name);
}
static void* (*_glewGetProcAddress)(const char*);

#ifdef _WIN32
static void* _glew_wgl_thunk_gpa(const char* name) {
  void* addr = _wglewGetProcAddress((LPCSTR)name);
  if (addr == NULL)
  {
    addr = GetProcAddress(dso_handle, (LPCSTR)name);
  }

  return addr;
}
#endif

/* Initialize our function loader.
 * This loads function pointers for .. functions that load function pointers.
 * This translates to glXGetProcAddress on X-based platforms, and OSMesa
 * otherwise. */
static int _glewInitFunctionLoader()
{
  int name_offset = (glew_convention == GLEW_NAME_MANGLED) ? 0 : 1;
#if defined(_WIN32)
  const char wgl_gpa[] = "mwglGetProcAddress";
  DWORD err = ERROR_SUCCESS;
  if (NULL == dso_handle)
  {
    dso_handle = GetModuleHandle(glew_gl_lib);
    if (dso_handle == NULL)
    {
      dso_handle = LoadLibrary(glew_gl_lib);
    }
    if (dso_handle == NULL)
    {
      error("Could not load library `%s': %d", glew_gl_lib, GetLastError());
      return GLEW_ERROR_DLOPEN;
    }
  }
  SetLastError(ERROR_SUCCESS);
#else
  const char *dl_error;
  const char glx_gpa[] = "mglXGetProcAddressARB";
  if(NULL == dso_handle) {
#ifdef _AIX
    /* AIX needs RTLD_MEMBER to load shared libs; otherwise it returns an
     * `invalid magic number' error.  This apparently can be worked around by
     * building the shared library in a special way, but the system's supplied
     * libGL.a requires this, at least. */
    dso_handle = dlopen(glew_gl_lib, RTLD_LAZY | RTLD_LOCAL | RTLD_MEMBER);
#else
    dso_handle = dlopen(glew_gl_lib, RTLD_LAZY | RTLD_LOCAL);
#endif
    if(NULL == dso_handle) {
      error("Could not load library `%s': %s", glew_gl_lib, dlerror());
      return GLEW_ERROR_DLOPEN;
    }
  }
  dlerror(); /* clear any previous error. */
#endif

  switch (gl_lib_type)
  {
    case GLEW_LIB_TYPE_NATIVE:
#ifdef _WIN32
      _wglewGetProcAddress = (WGLGPA)GetProcAddress(dso_handle,
                                                    wgl_gpa + name_offset);
      _glewGetProcAddress = _glew_wgl_thunk_gpa;
#elif defined(__APPLE__)
      __glewXGetProcAddress = NULL;
      _glewGetProcAddress = _glew_dlsym_thunk_gpa;
#else
      /* In Mesa 7.6 and earlier, glXGPA-ARB was not mangled.  In 7.6.1 and
       * later, it does get mangled.  We'll just try both; if the mangled
       * name exists, great.  Otherwise, fall back to the unmangled name.
       * Google: "glXGPA with mangled Mesa" for more information. */
      __glewXGetProcAddress = (GPA) dlsym(dso_handle, glx_gpa);
      if(__glewXGetProcAddress == NULL) {
        dlerror(); /* clear the error from loading the `mgl' variant. */
        __glewXGetProcAddress = (GPA) dlsym(dso_handle, glx_gpa + name_offset);
      }
      _glewGetProcAddress = _glew_glx_thunk_gpa;
#endif /* _WIN32 */
      break;
    case GLEW_LIB_TYPE_OSMESA:
#ifdef _WIN32
      _glewOSMesaGetProcAddress = (_GPA) GetProcAddress(dso_handle,
                                                        "OSMesaGetProcAddress");
#else
      _glewOSMesaGetProcAddress = (_GPA) dlsym(dso_handle,
                                               "OSMesaGetProcAddress");
#endif
      _glewGetProcAddress = _glew_osmesa_thunk_gpa;
      break;
  }

#ifdef _WIN32
  err = GetLastError();
  if (err != ERROR_SUCCESS)
  {
    error("Could not find dynamic loading function; %d", err);
    FreeLibrary(dso_handle);
    dso_handle = NULL;
    return GLEW_ERROR_GPA;
  }
#else
  if((dl_error = dlerror()) != NULL) {
    error("Could not find dynamic loading function: %s", dl_error);
    dlclose(dso_handle);
    dso_handle = NULL;
    return GLEW_ERROR_GPA;
  }
#endif
  return GLEW_NO_ERROR;
}

#if defined(__GNUC__) && __GNUC__ >= 4
/* If they've got a GNU compiler, install a DSO finalizer.  Useful for
 * valgrinding. */
#include <stdio.h>
__attribute__((destructor))
static void _glewUnloadDL()
{
    if(dso_handle != NULL) {
      dlclose(dso_handle);
      dso_handle = NULL;
    }
}
#endif

#endif /* __linux__ || __APPLE__ || _WIN32 */

#define fqn_from_convention(convention, fqn) \
    (convention == GLEW_NAME_MANGLED ? "m"fqn : fqn)

/*
* Define glewGetProcAddress.
*/
#if defined(_WIN32)
#  define glewGetProcAddress(name) (*_glewGetProcAddress)(name)
#else
#  if defined(__APPLE__) && defined(MAC_OS_X_VERSION_10_3) && !defined(MAC_OS_X_VERSION_10_4)
#    define glewGetProcAddress(name) NSGLGetProcAddress(name)
#  else
#    if defined(__sgi) || defined(__sun)
#      define glewGetProcAddress(name) dlGetProcAddress(name)
#    else /* __linux */
#      define glewGetProcAddress(name) (*_glewGetProcAddress)(name)
#    endif
#  endif
#endif

/*
* Define GLboolean const cast.
*/
#define CONST_CAST(x) (*(GLboolean*)&x)

/*
* GLEW, just like OpenGL or GLU, does not rely on the standard C library.
* These functions implement the functionality required in this file.
*/
static GLuint _glewStrLen (const GLubyte* s)
{
  GLuint i=0;
  if (s == NULL) return 0;
  while (s[i] != '\0') i++;
  return i;
}

static GLuint _glewStrCLen (const GLubyte* s, GLubyte c)
{
  GLuint i=0;
  if (s == NULL) return 0;
  while (s[i] != '\0' && s[i] != c) i++;
  return (s[i] == '\0' || s[i] == c) ? i : 0;
}

static GLboolean _glewStrSame (const GLubyte* a, const GLubyte* b, GLuint n)
{
  GLuint i=0;
  if(a == NULL || b == NULL)
    return (a == NULL && b == NULL && n == 0) ? GL_TRUE : GL_FALSE;
  while (i < n && a[i] != '\0' && b[i] != '\0' && a[i] == b[i]) i++;
  return i == n ? GL_TRUE : GL_FALSE;
}

static GLboolean _glewStrSame1 (GLubyte** a, GLuint* na, const GLubyte* b, GLuint nb)
{
  while (*na > 0 && (**a == ' ' || **a == '\n' || **a == '\r' || **a == '\t'))
  {
    (*a)++;
    (*na)--;
  }
  if(*na >= nb)
  {
    GLuint i=0;
    while (i < nb && (*a)+i != NULL && b+i != NULL && (*a)[i] == b[i]) i++;
    if(i == nb)
    {
      *a = *a + nb;
      *na = *na - nb;
      return GL_TRUE;
    }
  }
  return GL_FALSE;
}

static GLboolean _glewStrSame2 (GLubyte** a, GLuint* na, const GLubyte* b, GLuint nb)
{
  if(*na >= nb)
  {
    GLuint i=0;
    while (i < nb && (*a)+i != NULL && b+i != NULL && (*a)[i] == b[i]) i++;
    if(i == nb)
    {
      *a = *a + nb;
      *na = *na - nb;
      return GL_TRUE;
    }
  }
  return GL_FALSE;
}

static GLboolean _glewStrSame3 (GLubyte** a, GLuint* na, const GLubyte* b, GLuint nb)
{
  if(*na >= nb)
  {
    GLuint i=0;
    while (i < nb && (*a)+i != NULL && b+i != NULL && (*a)[i] == b[i]) i++;
    if (i == nb && (*na == nb || (*a)[i] == ' ' || (*a)[i] == '\n' || (*a)[i] == '\r' || (*a)[i] == '\t'))
    {
      *a = *a + nb;
      *na = *na - nb;
      return GL_TRUE;
    }
  }
  return GL_FALSE;
}
