/* ------------------------------------------------------------------------ */

const GLubyte* glewGetErrorString (GLenum error)
{
  static const GLubyte* _glewErrorString[] =
  {
    (const GLubyte*)"No error",
    (const GLubyte*)"Missing GL version",
    (const GLubyte*)"GL 1.1 and up are not supported",
    (const GLubyte*)"GLX 1.2 and up are not supported",
    (const GLubyte*)"GLX 1.1 and up are not supported",
    (const GLubyte*)"GLX not available",
    (const GLubyte*)"Library dlopen failed",
    (const GLubyte*)"Could not find appropriate GL function loader",
    (const GLubyte*)"Unknown error"
  };
  const int max_error = sizeof(_glewErrorString)/sizeof(*_glewErrorString) - 1;
  return _glewErrorString[(int)error > max_error ? max_error : (int)error];
}

const GLubyte* glewGetString (GLenum name)
{
  static const GLubyte* _glewString[] =
  {
    (const GLubyte*)NULL,
    (const GLubyte*)"GLEW_VERSION_STRING",
    (const GLubyte*)"GLEW_VERSION_MAJOR_STRING",
    (const GLubyte*)"GLEW_VERSION_MINOR_STRING",
    (const GLubyte*)"GLEW_VERSION_MICRO_STRING"
  };
  const int max_string = sizeof(_glewString)/sizeof(*_glewString) - 1;
  return _glewString[(int)name > max_string ? 0 : (int)name];
}

/* ------------------------------------------------------------------------ */

GLboolean glewExperimental = GL_FALSE;

#if !defined(GLEW_MX)

#if defined(_WIN32)
extern GLenum wglewContextInit (void);
#elif !defined(__APPLE__) || defined(GLEW_APPLE_GLX) /* _UNIX */
extern GLenum glxewContextInit (void);
#endif /* _WIN32 */

GLenum glewInitLibrary (const char *name, enum GL_Library_Type lib_type,
                        enum GL_Name_Convention name_conv)
{
  glew_gl_lib = name;
  gl_lib_type = lib_type;
  glew_convention = name_conv;

  return glewInit();
}

GLenum glewInit ()
{
  GLenum r;

  r = _glewInitFunctionLoader();
  if(GLEW_NO_ERROR != r) {
    return r;
  }

  r = glewContextInit();
#if defined(_WIN32)
  if(gl_lib_type != GLEW_LIB_TYPE_OSMESA) {
    return wglewContextInit();
  }
  return GLEW_OK;
#elif !defined(__APPLE__) || defined(GLEW_APPLE_GLX) /* _UNIX */
  /* We'll need GLX to setup contexts / etc. if we're not using OSMesa. */
  if(gl_lib_type != GLEW_LIB_TYPE_OSMESA) {
    return glxewContextInit();
  }
  return r;
#else
  return r;
#endif /* _WIN32 */
}

#endif /* !GLEW_MX */
